################################################################################
###
### Plot equilibrium CDFs and probability of victory in each of the four cases
###
################################################################################

library("dplyr")
library("ggplot2")
library("cowplot")
library("reshape2")
library("tikzDevice")
library("tidyr")

## Fix the following parameters throughout, and only vary alpha:
cc <- 2
p_A <- 0.4
p_D <- 1 - p_A

## Grid of values of spending to plot over
s_grid <- seq(0, 1, length.out = 101)

## Axis labels will be the same in each plot
x_label <- "Candidate spending"
y_label <- "Probability"

## Facet labels will be the same in each plot
facet_labels = c("Mixed strategy CDFs",
                 "Probability of victory")
add_facet_var <- function(x)
{
    mutate(x,
           type = factor(variable == "lambda",
                         levels = c(FALSE, TRUE),
                         labels = facet_labels))
}

## Legend labels will be the same in each plot:
##   D's mixed strategy
##   A's mixed strategy
##   Probability of victory
legend_title <- "Quantity"
legend_labels <- c(
    "F_D" = "$F_D^*(s)$",
    "F_A" = "$F_A^*(s)$",
    "lambda" = "$\\lambda(s)$"
)

## Dimensions will be the same for each plot
plot_width <- 5.5
plot_height <- 3.5
slides_width <- 4.25
slides_height <- 3

## Other common theming for plots
plot_line_size <- 1
plot_point_size <- 2.5
plot_rect_fill <- "gray90"

plot_common <- function(p)
{
    p +
        scale_y_continuous(y_label,
                           breaks = c(0, 1),
                           labels = c("$0$", "$1$")) +
        scale_linetype(legend_title,
                       labels = legend_labels) +
        scale_colour_brewer(legend_title,
                            labels = legend_labels,
                            palette = "Set1") +
        theme_cowplot(font_size = 11) +
        theme(legend.key.height = grid::unit(1.5, "line"),
              legend.key.width = grid::unit(0.5, "in"),
              axis.text = element_text(size = 9),
              axis.title = element_text(face = "plain", size = 10))
}

###-----------------------------------------------------------------------------
### Advantaged candidates in the majority
###-----------------------------------------------------------------------------

alpha_maj <- -1
stopifnot(alpha_maj <= 0)

## Strategy cutpoints
s_D_bar_maj <- p_D / cc
s_A_bar_maj <- s_D_bar_maj + p_A

## Equilibrium quantities
dat_maj <- data.frame(s = s_grid) %>%
    mutate(F_D = ifelse(s <= s_D_bar_maj,
                        cc * s / p_D,
                        1),
           F_A = ifelse(s <= s_D_bar_maj,
                        0,
                 ifelse(s > s_A_bar_maj,
                        1,
                        (s - s_D_bar_maj) / p_A)),
           lambda = p_D * F_D + p_A * F_A) %>%
    melt(id.vars = "s") %>%
    add_facet_var()

## Both types employ continuous mixed strategies, so plotting is easy
plot_maj <- ggplot(dat_maj, aes(x = s, y = value)) +
    geom_line(aes(colour = variable,
                  linetype = variable),
              size = plot_line_size) +
    facet_wrap(~ type, nrow = 2) +
    ggtitle("Equilibrium in Proposition~1") +
    scale_x_continuous(x_label,
                       limits = c(0, s_A_bar_maj),
                       breaks = c(0, s_D_bar_maj, s_A_bar_maj),
                       labels = c("$0$",
                                  "$\\bar{s}_D^*$",
                                  "$\\bar{s}_A^*$"))
plot_maj <- plot_common(plot_maj)

tikz(file = "figure-1.tex",
     width = plot_width,
     height = plot_height)
print(plot_maj)
dev.off()


###-----------------------------------------------------------------------------
### Advantaged candidates in the minority, medium alpha
###-----------------------------------------------------------------------------

alpha_med <- 0.4
stopifnot(alpha_med < 1 / (2 * p_D) &&
          alpha_med >= p_D / 2)

## Equilibrium mixing probability and cutpoints
pi_med <- (sqrt(2 * alpha_med * p_D) - p_D) / p_A
s_A_tilde_med <- (pi_med * p_A + p_D) / 2
s_A_bar_med <- 1 - s_A_tilde_med

## Equilibrium quantities
dat_med <- data.frame(s = s_grid) %>%
    mutate(F_D = 1,
           F_A = ifelse(s <= s_A_tilde_med,
                        pi_med,
                 ifelse(s <= s_A_bar_med,
                        pi_med + (s - s_A_tilde_med) / p_A,
                        1)),
           lambda = ifelse(s <= s_A_tilde_med,
                           0,
                           p_A * F_A + p_D)) %>%
    melt(id.vars = "s") %>%
    filter(variable != "lambda" | value > 0) %>%
    add_facet_var()

filled_med <- rbind(
    data.frame(s = 0, variable = "F_D", value = 1),
    data.frame(s = 0, variable = "F_A", value = pi_med),
    data.frame(s = 0, variable = "lambda", value = s_A_tilde_med),
    data.frame(s = s_A_tilde_med, variable = "lambda", value = 2 * s_A_tilde_med)
) %>% add_facet_var()

empty_med <- rbind(
    data.frame(s = 0, variable = "lambda", value = 0),
    data.frame(s = s_A_tilde_med, variable = "lambda", value = 0)
) %>% add_facet_var()

seg_med <- data.frame(s = 0,
                      s1 = s_A_tilde_med,
                      value = 0,
                      v1 = 0,
                      variable = "lambda") %>%
    add_facet_var()

plot_med <- ggplot(dat_med,
                   aes(x = s,
                       y = value,
                       colour = variable,
                       linetype = variable)) +
    geom_rect(aes(colour = NULL,
                  linetype = NULL),
              xmin = 0,
              xmax = s_A_tilde_med,
              ymin = 0,
              ymax = 1,
              colour = "white",
              fill = plot_rect_fill,
              show_guide = FALSE) +
    geom_line(size = plot_line_size) +
    geom_segment(aes(xend = s1, yend = v1),
                 data = seg_med,
                 size = plot_line_size,
                 show_guide = FALSE) +
    geom_point(data = filled_med,
               size = plot_point_size,
               show_guide = FALSE) +
    geom_point(data = empty_med,
               shape = 21,
               size = plot_point_size,
               fill = "white",
               show_guide = FALSE) +
    facet_wrap(~ type, nrow = 2) +
    ggtitle("Equilibrium in Proposition~3") +
    scale_x_continuous(x_label,
                       limits = c(0, s_A_bar_med),
                       breaks = c(0,
                                  s_A_tilde_med,
                                  s_A_bar_med),
                       labels = c("$0$",
                                  "$\\tilde{s}_A^*$",
                                  "$\\bar{s}_A^*$"))
plot_med <- plot_common(plot_med)

tikz(file = "figure-2.tex",
     width = plot_width,
     height = plot_height)
print(plot_med)
dev.off()

###-----------------------------------------------------------------------------
### Advantaged candidates in the minority, low alpha (referred to as the "high"
### case for legacy reasons)
###-----------------------------------------------------------------------------

alpha_high <- 0.15
stopifnot(alpha_high < p_D / 2)

## Equilibrium cutpoints
s_D_tilde_high <- (p_D - 2 * alpha_high) / cc
s_D_bar_high <- (p_D - alpha_high) / cc
s_A_ubar_high <- (p_D + (cc - 1) * alpha_high) / cc
s_A_bar_high <- s_A_ubar_high + p_A

## Additional conditions for plotting convenience
stopifnot(s_D_bar_high > alpha_high)
stopifnot(s_A_ubar_high > s_D_tilde_high + alpha_high)
stopifnot(s_D_bar_high < s_D_tilde_high + alpha_high)

## Line segments to plot
seg_high <- rbind(
    data.frame(smin = 0,
               smax = s_D_tilde_high,
               vmin = 0,
               vmax = cc * s_D_tilde_high / p_D,
               variable = "F_D"),
    data.frame(smin = s_D_tilde_high,
               smax = s_D_bar_high,
               vmin = cc * s_D_tilde_high / p_D,
               vmax = cc * s_D_tilde_high / p_D,
               variable = "F_D"),
    data.frame(smin = s_D_bar_high,
               smax = s_A_bar_high,
               vmin = 1,
               vmax = 1,
               variable = "F_D"),
    data.frame(smin = 0,
               smax = s_A_ubar_high,
               vmin = 0,
               vmax = 0,
               variable = "F_A"),
    data.frame(smin = s_A_ubar_high,
               smax = s_A_bar_high,
               vmin = 0,
               vmax = 1,
               variable = "F_A"),
    data.frame(smin = 0,
               smax = s_D_tilde_high,
               vmin = 0,
               vmax = cc * s_D_tilde_high,
               variable = "lambda"),
    data.frame(smin = s_D_tilde_high,
               smax = s_D_bar_high,
               vmin = cc * s_D_tilde_high,
               vmax = cc * s_D_tilde_high,
               variable = "lambda"),
    data.frame(smin = s_D_bar_high,
               smax = s_D_tilde_high + alpha_high,
               vmin = cc * (s_D_bar_high - alpha_high),
               vmax = cc * s_D_tilde_high,
               variable = "lambda"),
    data.frame(smin = s_D_tilde_high + alpha_high,
               smax = s_A_ubar_high,
               vmin = cc * s_D_tilde_high,
               vmax = cc * s_D_tilde_high,
               variable = "lambda"),
    data.frame(smin = s_A_ubar_high,
               smax = s_A_bar_high,
               vmin = p_D,
               vmax = 1,
               variable = "lambda")
) %>% add_facet_var()

## Filled points to plot
filled_high <- rbind(
    data.frame(s = s_D_bar_high,
               v = 1,
               variable = "F_D"),
    data.frame(s = s_D_bar_high,
               v = p_D - alpha_high,
               variable = "lambda"),
    data.frame(s = s_A_ubar_high,
               v = p_D,
               variable = "lambda")
) %>% add_facet_var()

## Empty points to plot
empty_high <- rbind(
    data.frame(s = s_D_bar_high,
               v = cc * s_D_tilde_high / p_D,
               variable = "F_D"),
    data.frame(s = s_D_bar_high,
               v = cc * s_D_tilde_high,
               variable = "lambda"),
    data.frame(s = s_D_bar_high,
               v = cc * (s_D_bar_high - alpha_high),
               variable = "lambda"),
    data.frame(s = s_A_ubar_high,
               v = cc * s_D_tilde_high,
               variable = "lambda")
) %>% add_facet_var()

plot_high <- ggplot(seg_high,
                    aes(colour = variable,
                        linetype = variable)) +
    geom_rect(aes(colour = NULL,
                  linetype = NULL),
              xmin = s_D_tilde_high,
              xmax = s_D_bar_high,
              ymin = 0,
              ymax = 1,
              colour = "white",
              fill = plot_rect_fill,
              show_guide = FALSE) +
    geom_rect(aes(colour = NULL,
                  linetype = NULL),
              xmin = s_D_bar_high,
              xmax = s_A_ubar_high,
              ymin = 0,
              ymax = 1,
              colour = "white",
              fill = plot_rect_fill,
              show_guide = FALSE) +
    geom_segment(aes(x = smin,
                     xend = smax,
                     y = vmin,
                     yend = vmax),
                 size = plot_line_size) +
    geom_point(aes(x = s,
                   y = v),
               data = filled_high,
               size = plot_point_size,
               show_guide = FALSE) +
    geom_point(aes(x = s,
                   y = v),
               data = empty_high,
               size = plot_point_size,
               shape = 21,
               fill = "white",
               show_guide = FALSE) +
    facet_wrap(~ type, nrow = 2) +
    ggtitle("Equilibrium in Proposition~4") +
    scale_x_continuous(x_label,
                       limits = c(0, s_A_bar_high),
                       breaks = c(0,
                                  s_D_tilde_high,
                                  s_D_bar_high,
                                  s_A_ubar_high,
                                  s_A_bar_high),
                       labels = c("$0$",
                                  "$\\tilde{s}_D^*$",
                                  "$\\bar{s}_D^*$",
                                  "$\\underaccent{\\bar}{s}_A^*$",
                                  "$\\bar{s}_A^*$"))
plot_high <- plot_common(plot_high)

tikz(file = "figure-3.tex",
     width = plot_width,
     height = plot_height,
     packages = c(getOption("tikzLatexPackages"),
                  "\\usepackage{accents}"))
print(plot_high)
dev.off()


## ----------------------------------------------------------------------
## Plot ex ante victory probability as function of parameters
## ----------------------------------------------------------------------

p_A <- 0.5
p_D <- 0.5

c_A_alpha <- seq(-1, 1.5, length.out = 201)
pi_star <- (sqrt(2 * c_A_alpha * p_D) - p_D) / p_A
prob <- ifelse(
    c_A_alpha <= p_D / 2,
    1 - p_D^2,
        ifelse(c_A_alpha < 1 / (2 * p_D),
               p_A^2 + 2 * p_A * p_D * (1 - pi_star / 2),
               p_A^2 + p_A * p_D))

plot_data_ex_ante <- data_frame(
    c_A_alpha = c_A_alpha,
    prob = prob
)

plot_ex_ante <- ggplot(plot_data_ex_ante, aes(x = c_A_alpha, y = prob)) +
    geom_line(size = plot_line_size) +
    geom_vline(xintercept = c(0),
               linetype = 3) +
    annotate(geom = "text",
             x = -0.5,
             y = 0.775,
             label = "$A$ centrist",
             size = 3) +
    annotate(geom = "text",
             x = 0.75,
             y = 0.775,
             label = "$A$ non-centrist",
             size = 3) +
    scale_x_continuous("$c_A \\alpha$",
                       breaks = c(0, p_D / 2, 1 / (2 * p_D)),
                       labels = c("0", "$p_D / 2$", "$1 / 2 p_D$")) +
    scale_y_continuous("Probability winner's type is $A$",
                       breaks = c(0,
                                  1 - p_A * p_D - p_D^2,
                                  1 - p_D^2,
                                  1),
                       labels = c("0",
                                  "$1 - p_A p_D - p_D^2$",
                                  "$1 - p_D^2$",
                                  "1")) +
    theme_cowplot(font_size = 11) +
    theme(legend.key.height = grid::unit(1.5, "line"),
          legend.key.width = grid::unit(0.5, "in"),
          axis.text = element_text(size = 9),
          axis.title = element_text(face = "plain", size = 10),
          axis.title.x = element_text(margin = margin(10, 0, 0, 0)))

tikz(file = "figure-4.tex",
     width = plot_width,
     height = 3,
     packages = c(getOption("tikzLatexPackages"),
                  "\\usepackage{accents}"))
print(plot_ex_ante)
dev.off()


## -----------------------------------------------------------------------------
## Plot median voter's ex ante expected utility
## -----------------------------------------------------------------------------

## Set parameters
p_D <- 0.5
p_A <- 1 - p_D
alpha <- 1
c_A <- seq(0.1, 1.5, length.out = 100)
c_D <- 2
stopifnot(c_D > max(c_A))
pi_star <- (sqrt(2 * alpha * c_A * p_D) - p_D) / p_A
ubar_s_A <- alpha + (p_D - c_A * alpha) / c_D
bar_s_A <- ubar_s_A + (p_A / c_A)

## Calculate utilities in each case
Eu_m_low <- p_A * -1 * alpha
Eu_m_med <- 2 * p_A * p_D * (
    (1 - pi_star) / (2 * c_A) - alpha * (1 - pi_star / 2)
) + p_A^2 * (
    (1 - pi_star^2) * (4 - p_D - pi_star * p_A) / (6 * c_A) - alpha
)
Eu_m_high <- p_D^2 * (
    2 * (p_D^3 - 2 * alpha^3 * c_A^3) / (3 * c_D * p_D^2)
) + 2 * p_A * p_D * (
    (1/2) * ubar_s_A + (1/2) * bar_s_A
) + p_A^2 * (
    (1/3) * ubar_s_A + (2/3) * bar_s_A
)

## When Advantaged candidates are centrist
U_m_extremist <- case_when(
    alpha <= p_D / (2 * c_A) ~ Eu_m_high,
    alpha < 1 / (2 * c_A * p_D) ~ Eu_m_med,
    TRUE ~ Eu_m_low
)

## When Advantaged candidates are non-centrist
U_m_centrist <- p_D^2 * (
    (2/3) * p_D / c_D
) + 2 * p_A * p_D * (
    (1/2) * p_D / c_D + (1/2) * (p_D / c_D + p_A / c_A)
) + p_A^2 * (
    (1/3) * p_D / c_D + (2/3) * (p_D / c_D + p_A / c_A)
)

plot_data_welfare <- data_frame(c_A, U_m_centrist, U_m_extremist)

plot_welfare_centrist <-
    ggplot(plot_data_welfare, aes(x = c_A, y = U_m_centrist)) +
    geom_line(size = plot_line_size) +
    scale_x_continuous("$c_A$",
                       breaks = c(0, p_A, 2 * p_A),
                       labels = c("0", "$p_A$", "$2 p_A$")) +
    scale_y_continuous("Median voter's expected utility") +
    theme_cowplot(font_size = 11) +
    theme(legend.key.height = grid::unit(1.5, "line"),
          legend.key.width = grid::unit(0.5, "in"),
          axis.text = element_text(size = 9),
          axis.title = element_text(face = "plain", size = 10),
          axis.title.x = element_text(margin = margin(10, 0, 0, 0)),
          axis.ticks.y = element_blank(),
          axis.text.y = element_blank())

plot_welfare_extremist <-
    ggplot(plot_data_welfare %>% filter(c_A <= p_D / (2 * alpha)),
           aes(x = c_A, y = U_m_extremist)) +
    geom_line(size = plot_line_size) +
    geom_line(data = plot_data_welfare %>% filter(c_A > p_D / (2 * alpha)),
              mapping = aes(y = U_m_extremist),
              size = plot_line_size) +
    scale_x_continuous("$c_A$",
                       breaks = c(0, p_D / (2 * alpha), 1 / (2 * p_D * alpha)),
                       labels = c("0", "$p_D / 2 \\alpha$", "$1 / 2 p_D \\alpha$")) +
    scale_y_continuous("Median voter's expected utility") +
    theme_cowplot(font_size = 11) +
    theme(legend.key.height = grid::unit(1.5, "line"),
          legend.key.width = grid::unit(0.5, "in"),
          axis.text = element_text(size = 9),
          axis.title = element_text(face = "plain", size = 10),
          axis.title.x = element_text(margin = margin(10, 0, 0, 0)),
          axis.ticks.y = element_blank(),
          axis.text.y = element_blank())

tikz(file = "figure-5a.tex",
     width = 0.5 * plot_width,
     height = 2.5)
print(plot_welfare_centrist)
dev.off()

tikz(file = "figure-5b.tex",
     width = 0.5 * plot_width,
     height = 2.5)
print(plot_welfare_extremist)
dev.off()


## -----------------------------------------------------------------------------
## Plot equilibrium expected utility of candidates
## -----------------------------------------------------------------------------

## Set parameters
p_D <- 0.75
p_A <- 1 - p_D
alpha <- 0.75
c_A <- seq(0.1, 1.5, length.out = 100)
c_D <- 2
stopifnot(c_D > max(c_A))

## When centrists are Advantaged
uD_centrist <- rep(0, length(c_A))
uA_centrist <- p_D - c_A * (p_D / c_D)

## When non-centrists are Advantaged
pi_star <- (sqrt(2 * alpha * c_A * p_D) - p_D) / p_A
bar_s_D <- (p_D - c_A * alpha) / c_D
uD_extremist <- case_when(
    alpha >= 1 / (2 * c_A * p_D) ~
        1/2,
    alpha > p_D / (2 * c_A) ~
        (1/2) * (p_D + pi_star * p_A),
    TRUE ~
        0
)
uA_extremist <- case_when(
    alpha >= 1 / (2 * c_A * p_D) ~
        1/2,
    alpha > p_D / (2 * c_A) ~
        (1/2) * (p_D + pi_star * p_A),
    TRUE ~
        (c_D - c_A) * bar_s_D
)

plot_data_candwelf <- data_frame(c_A,
                                 uD_centrist,
                                 uA_centrist,
                                 uD_extremist,
                                 uA_extremist) %>%
    gather(key, value, -c_A) %>%
    separate(key, into = c("Type", "model")) %>%
    mutate(Type = recode(Type, uA = "$A$", uD = "$D$"))

plot_candwelf_centrist <-
    ggplot(plot_data_candwelf %>% filter(model == "centrist"),
       aes(x = c_A, y = value)) +
    geom_line(aes(linetype = Type, colour = Type),
              size = plot_line_size) +
    scale_x_continuous("$c_A$",
                       breaks = c(0, 2 * p_A, 4 * p_A),
                       labels = c("0", "$p_A$", "$2 p_A$")) +
    scale_y_continuous("Candidate's expected utility") +
    scale_colour_brewer(palette = "Set1") +
    theme_cowplot(font_size = 11) +
    theme(legend.key.height = grid::unit(1.5, "line"),
          legend.key.width = grid::unit(0.5, "in"),
          legend.position = "bottom",
          axis.text = element_text(size = 9),
          axis.title = element_text(face = "plain", size = 10),
          axis.title.x = element_text(margin = margin(10, 0, 0, 0)),
          axis.ticks.y = element_blank(),
          axis.text.y = element_blank())

tikz(file = "figure-6a.tex",
     width = 0.5 * plot_width,
     height = 3)
print(plot_candwelf_centrist)
dev.off()

plot_candwelf_extremist <-
    ggplot(plot_data_candwelf %>% filter(model == "extremist", c_A <= p_D / (2 * alpha)),
       aes(x = c_A, y = value)) +
    geom_line(aes(linetype = Type, colour = Type),
              size = plot_line_size) +
    geom_line(data = plot_data_candwelf %>% filter(model == "extremist", c_A > p_D / (2 * alpha)),
              aes(linetype = Type, colour = Type),
              size = plot_line_size) +
    scale_x_continuous("$c_A$",
                       breaks = c(0, p_D / (2 * alpha), 1 / (2 * p_D * alpha)),
                       labels = c("0", "$p_D / 2 \\alpha$", "$1 / 2 p_D \\alpha$")) +
    scale_y_continuous("Candidate's expected utility") +
    scale_colour_brewer(palette = "Set1") +
    theme_cowplot(font_size = 11) +
    theme(legend.key.height = grid::unit(1.5, "line"),
          legend.key.width = grid::unit(0.5, "in"),
          legend.position = "bottom",
          axis.text = element_text(size = 9),
          axis.title = element_text(face = "plain", size = 10),
          axis.title.x = element_text(margin = margin(10, 0, 0, 0)),
          axis.ticks.y = element_blank(),
          axis.text.y = element_blank())

tikz(file = "figure-6b.tex",
     width = 0.5 * plot_width,
     height = 3)
print(plot_candwelf_extremist)
dev.off()
